@@ -384,6 +384,7 @@ pub const BTN_TOOL_FINGER:    c_int = 0x145;
 pub const BTN_TOOL_MOUSE:     c_int = 0x146;
 pub const BTN_TOOL_LENS:      c_int = 0x147;
 pub const BTN_TOOL_QUINTTAP:  c_int = 0x148; /* Five fingers on trackpad */
+pub const BTN_STYLUS3:        c_int = 0x149;
 pub const BTN_TOUCH:          c_int = 0x14a;
 pub const BTN_STYLUS:         c_int = 0x14b;
 pub const BTN_STYLUS2:        c_int = 0x14c;
@@ -415,10 +416,12 @@ pub const KEY_LANGUAGE:          c_int = 0x170;
 pub const KEY_TITLE:             c_int = 0x171;
 pub const KEY_SUBTITLE:          c_int = 0x172;
 pub const KEY_ANGLE:             c_int = 0x173;
-pub const KEY_ZOOM:              c_int = 0x174;
+pub const KEY_FULL_SCREEN:       c_int = 0x174; /* AC View Toggle */
+pub const KEY_ZOOM:              c_int = KEY_FULL_SCREEN;
 pub const KEY_MODE:              c_int = 0x175;
 pub const KEY_KEYBOARD:          c_int = 0x176;
-pub const KEY_SCREEN:            c_int = 0x177;
+pub const KEY_ASPECT_RATIO:      c_int = 0x177; /* HUTRR37: Aspect */
+pub const KEY_SCREEN:            c_int = KEY_ASPECT_RATIO;
 pub const KEY_PC:                c_int = 0x178; /* Media Select Computer */
 pub const KEY_TV:                c_int = 0x179; /* Media Select TV */
 pub const KEY_TV2:               c_int = 0x17a; /* Media Select Cable */
@@ -569,15 +572,18 @@ pub const BTN_DPAD_DOWN:  c_int = 0x221;
 pub const BTN_DPAD_LEFT:  c_int = 0x222;
 pub const BTN_DPAD_RIGHT: c_int = 0x223;
 
-pub const KEY_ALS_TOGGLE: c_int = 0x230; /* Ambient light sensor */
+pub const KEY_ALS_TOGGLE:         c_int = 0x230; /* Ambient light sensor */
+pub const KEY_ROTATE_LOCK_TOGGLE: c_int = 0x231; /* Display rotation lock */
 
-pub const KEY_BUTTONCONFIG: c_int = 0x240;  /* AL Button Configuration */
-pub const KEY_TASKMANAGER:  c_int = 0x241;  /* AL Task/Project Manager */
-pub const KEY_JOURNAL:      c_int = 0x242;  /* AL Log/Journal/Timecard */
-pub const KEY_CONTROLPANEL: c_int = 0x243;  /* AL Control Panel */
-pub const KEY_APPSELECT:    c_int = 0x244;  /* AL Select Task/Application */
-pub const KEY_SCREENSAVER:  c_int = 0x245;  /* AL Screen Saver */
-pub const KEY_VOICECOMMAND: c_int = 0x246;  /* Listening Voice Command */
+pub const KEY_BUTTONCONFIG:    c_int = 0x240;  /* AL Button Configuration */
+pub const KEY_TASKMANAGER:     c_int = 0x241;  /* AL Task/Project Manager */
+pub const KEY_JOURNAL:         c_int = 0x242;  /* AL Log/Journal/Timecard */
+pub const KEY_CONTROLPANEL:    c_int = 0x243;  /* AL Control Panel */
+pub const KEY_APPSELECT:       c_int = 0x244;  /* AL Select Task/Application */
+pub const KEY_SCREENSAVER:     c_int = 0x245;  /* AL Screen Saver */
+pub const KEY_VOICECOMMAND:    c_int = 0x246;  /* Listening Voice Command */
+pub const KEY_ASSISTANT:       c_int = 0x247;  /* AL Context-aware desktop assistant */
+pub const KEY_KBD_LAYOUT_NEXT: c_int = 0x248; /* AC Next Keyboard Layout Select */
 
 pub const KEY_BRIGHTNESS_MIN: c_int = 0x250; /* Set Brightness to Minimum */
 pub const KEY_BRIGHTNESS_MAX: c_int = 0x251; /* Set Brightness to Maximum */
@@ -620,6 +626,84 @@ pub const KEY_SLOWREVERSE: c_int = 0x276;
  */
 pub const KEY_DATA: c_int = 0x277;
 pub const KEY_ONSCREEN_KEYBOARD: c_int = 0x278;
+pub const KEY_PRIVACY_SCREEN_TOGGLE: c_int = 0x279; /* Electronic privacy screen control */
+
+pub const KEY_SELECTIVE_SCREENSHOT: c_int = 0x27a; /* Select an area of screen to be copied */
+
+/*
+ * Some keyboards have keys which do not have a defined meaning, these keys
+ * are intended to be programmed / bound to macros by the user. For most
+ * keyboards with these macro-keys the key-sequence to inject, or action to
+ * take, is all handled by software on the host side. So from the kernel's
+ * point of view these are just normal keys.
+ *
+ * The KEY_MACRO# codes below are intended for such keys, which may be labeled
+ * e.g. G1-G18, or S1 - S30. The KEY_MACRO# codes MUST NOT be used for keys
+ * where the marking on the key does indicate a defined meaning / purpose.
+ *
+ * The KEY_MACRO# codes MUST also NOT be used as fallback for when no existing
+ * KEY_FOO define matches the marking / purpose. In this case a new KEY_FOO
+ * define MUST be added.
+ */
+pub const KEY_MACRO1:  c_int = 0x290;
+pub const KEY_MACRO2:  c_int = 0x291;
+pub const KEY_MACRO3:  c_int = 0x292;
+pub const KEY_MACRO4:  c_int = 0x293;
+pub const KEY_MACRO5:  c_int = 0x294;
+pub const KEY_MACRO6:  c_int = 0x295;
+pub const KEY_MACRO7:  c_int = 0x296;
+pub const KEY_MACRO8:  c_int = 0x297;
+pub const KEY_MACRO9:  c_int = 0x298;
+pub const KEY_MACRO10: c_int = 0x299;
+pub const KEY_MACRO11: c_int = 0x29a;
+pub const KEY_MACRO12: c_int = 0x29b;
+pub const KEY_MACRO13: c_int = 0x29c;
+pub const KEY_MACRO14: c_int = 0x29d;
+pub const KEY_MACRO15: c_int = 0x29e;
+pub const KEY_MACRO16: c_int = 0x29f;
+pub const KEY_MACRO17: c_int = 0x2a0;
+pub const KEY_MACRO18: c_int = 0x2a1;
+pub const KEY_MACRO19: c_int = 0x2a2;
+pub const KEY_MACRO20: c_int = 0x2a3;
+pub const KEY_MACRO21: c_int = 0x2a4;
+pub const KEY_MACRO22: c_int = 0x2a5;
+pub const KEY_MACRO23: c_int = 0x2a6;
+pub const KEY_MACRO24: c_int = 0x2a7;
+pub const KEY_MACRO25: c_int = 0x2a8;
+pub const KEY_MACRO26: c_int = 0x2a9;
+pub const KEY_MACRO27: c_int = 0x2aa;
+pub const KEY_MACRO28: c_int = 0x2ab;
+pub const KEY_MACRO29: c_int = 0x2ac;
+pub const KEY_MACRO30: c_int = 0x2ad;
+
+/*
+ * Some keyboards with the macro-keys described above have some extra keys
+ * for controlling the host-side software responsible for the macro handling:
+ * -A macro recording start/stop key. Note that not all keyboards which emit
+ *  KEY_MACRO_RECORD_START will also emit KEY_MACRO_RECORD_STOP if
+ *  KEY_MACRO_RECORD_STOP is not advertised, then KEY_MACRO_RECORD_START
+ *  should be interpreted as a recording start/stop toggle;
+ * -Keys for switching between different macro (pre)sets, either a key for
+ *  cycling through the configured presets or keys to directly select a preset.
+ */
+pub const KEY_MACRO_RECORD_START: c_int = 0x2b0;
+pub const KEY_MACRO_RECORD_STOP:  c_int = 0x2b1;
+pub const KEY_MACRO_PRESET_CYCLE: c_int = 0x2b2;
+pub const KEY_MACRO_PRESET1:      c_int = 0x2b3;
+pub const KEY_MACRO_PRESET2:      c_int = 0x2b4;
+pub const KEY_MACRO_PRESET3:      c_int = 0x2b5;
+
+/*
+ * Some keyboards have a buildin LCD panel where the contents are controlled
+ * by the host. Often these have a number of keys directly below the LCD
+ * intended for controlling a menu shown on the LCD. These keys often don't
+ * have any labeling so we just name them KEY_KBD_LCD_MENU#
+ */
+pub const KEY_KBD_LCD_MENU1: c_int = 0x2b8;
+pub const KEY_KBD_LCD_MENU2: c_int = 0x2b9;
+pub const KEY_KBD_LCD_MENU3: c_int = 0x2ba;
+pub const KEY_KBD_LCD_MENU4: c_int = 0x2bb;
+pub const KEY_KBD_LCD_MENU5: c_int = 0x2bc;
 
 pub const BTN_TRIGGER_HAPPY:   c_int = 0x2c0;
 pub const BTN_TRIGGER_HAPPY1:  c_int = 0x2c0;
@@ -672,18 +756,28 @@ pub const KEY_CNT:             c_int = KEY_MAX + 1;
  * Relative axes
  */
 
-pub const REL_X:      c_int = 0x00;
-pub const REL_Y:      c_int = 0x01;
-pub const REL_Z:      c_int = 0x02;
-pub const REL_RX:     c_int = 0x03;
-pub const REL_RY:     c_int = 0x04;
-pub const REL_RZ:     c_int = 0x05;
-pub const REL_HWHEEL: c_int = 0x06;
-pub const REL_DIAL:   c_int = 0x07;
-pub const REL_WHEEL:  c_int = 0x08;
-pub const REL_MISC:   c_int = 0x09;
-pub const REL_MAX:    c_int = 0x0f;
-pub const REL_CNT:    c_int = REL_MAX + 1;
+pub const REL_X:             c_int = 0x00;
+pub const REL_Y:             c_int = 0x01;
+pub const REL_Z:             c_int = 0x02;
+pub const REL_RX:            c_int = 0x03;
+pub const REL_RY:            c_int = 0x04;
+pub const REL_RZ:            c_int = 0x05;
+pub const REL_HWHEEL:        c_int = 0x06;
+pub const REL_DIAL:          c_int = 0x07;
+pub const REL_WHEEL:         c_int = 0x08;
+pub const REL_MISC:          c_int = 0x09;
+/*
+ * 0x0a is reserved and should not be used in input drivers.
+ * It was used by HID as REL_MISC+1 and userspace needs to detect if
+ * the next REL_* event is correct or is just REL_MISC + n.
+ * We define here REL_RESERVED so userspace can rely on it and detect
+ * the situation described above.
+ */
+pub const REL_RESERVED:      c_int = 0x0a;
+pub const REL_WHEEL_HI_RES:  c_int = 0x0b;
+pub const REL_HWHEEL_HI_RES: c_int = 0x0c;
+pub const REL_MAX:           c_int = 0x0f;
+pub const REL_CNT:           c_int = REL_MAX + 1;
 
 /*
  * Absolute axes
@@ -718,6 +812,15 @@ pub const ABS_VOLUME: c_int = 0x20;
 
 pub const ABS_MISC: c_int = 0x28;
 
+/*
+ * 0x2e is reserved and should not be used in input drivers.
+ * It was used by HID as ABS_MISC+6 and userspace needs to detect if
+ * the next ABS_* event is correct or is just ABS_MISC + n.
+ * We define here ABS_RESERVED so userspace can rely on it and detect
+ * the situation described above.
+ */
+pub const ABS_RESERVED: c_int = 0x2e;
+
 pub const ABS_MT_SLOT:        c_int = 0x2f; /* MT slot being modified */
 pub const ABS_MT_TOUCH_MAJOR: c_int = 0x30; /* Major axis of touching ellipse */
 pub const ABS_MT_TOUCH_MINOR: c_int = 0x31; /* Minor axis : c_int = omit if circular */
@@ -759,7 +862,8 @@ pub const SW_ROTATE_LOCK:          c_int = 0x0c;  /* set = rotate locked/disable
 pub const SW_LINEIN_INSERT:        c_int = 0x0d;  /* set = inserted */
 pub const SW_MUTE_DEVICE:          c_int = 0x0e;  /* set = device disabled */
 pub const SW_PEN_INSERTED:         c_int = 0x0f;  /* set = pen inserted */
-pub const SW_MAX:                  c_int = 0x0f;
+pub const SW_MACHINE_COVER:        c_int = 0x10;  /* set = cover closed */
+pub const SW_MAX:                  c_int = 0x10;
 pub const SW_CNT:                  c_int = SW_MAX + 1;
 
 /*
